#include "uart1.h"

u8  COMD485[1] = {0xaa};			//帧头
static u8 RecData;							//用于存放接收到的数据
u8  TX1_Cnt;    //发送计数,发送数据中, 用于记录已经发送到第几个字节
u8  RX1_Cnt;    //接收计数,记录接收数据个数
static u8 *pSendData;							//指向需要发送数据包的首地址
static u8 SendLength; 						//用于记录发送数据包的字节个数
bit B_TX1_Busy; //发送忙标志
//u8 idata RX1_Buffer[3]; //接收缓冲,用于存放接收到的数据
u8 RX1_Buffer[3];	
//========================================================================
// 函数: void TX1_data()
// 描述: 串口1发送数据函数
// 参数: tx1dat: 要发送的数据
// 返回: none
// 备注: 
//========================================================================
void TX1_data(u8 tx1dat)
{
	SBUF = tx1dat;
	while(B_TX1_Busy);
	B_TX1_Busy = 1;	
	//while(!TI);
	//TI = 0;
}
//========================================================================
// 函数: void WriteUart(u8 *pData ,u8 Length)
// 描述: 将数据写入串口
// 参数: *pData ：写入数据的首地址;Length：写入数据的长度
// 返回: none
// 备注: 
//========================================================================
void WriteUart(u8 *pData ,u8 Length)
{
	RS485_TX();
	if(Length == 0)
	{
		return;
	}
	if(B_TX1_Busy == 1)
		return;
	SendLength = 	Length;
	pSendData = pData;
	SBUF = *pSendData;
	B_TX1_Busy = 1;
	TX1_Cnt = 0;
}
//========================================================================
// 函数: u8 *ReadUart(void)
// 描述: 读取串口数据
// 参数: 无
// 返回: 返回串口数据的值
// 备注: 
//========================================================================
u8 *ReadUart(void)
{
	#if 1
	//RS485_RX();
	return RX1_Buffer; 
	#else
	u8 RData;
	RS485_RX();
	RData = RecData;
	RecData = 0;	
	return RData;	
	#endif
}
//========================================================================
// 函数: void PrintString1(u8 *puts)
// 描述: 串口1发送字符串函数
// 参数: puts:  字符串指针
// 返回: none
// 备注: 
//========================================================================
void PrintString1(u8 *puts)
{
	#if 1
	for (; *puts != 0;  puts++)     //到停止符0结束
	{
		TX1_data(*puts);
	}
	#else
	while(*puts)
		TX1_data(*puts++);
	#endif
}

//========================================================================
// 函数: void UART1_config(u32 FOSC,u32 BAUD1)
// 描述: UART1初始化函数
// 参数: FOSC:设置晶振时钟频率，BAUD1:设置波特率
// 返回: none
// 备注: 
//========================================================================
void UART1_config(u32 FOSC,u32 BAUD1)
{
	P_SW1 |= 0x80;      //0x00: P3.0 P3.1, 0x40: P3.6 P3.7, 0x80: P1.6 P1.7 (必须使用内部时钟)
	TR1 = 0;
	AUXR &= ~0x01;      //S1 BRT Use Timer1; B0位是timer选择位，0选timer1,1选timer2
	AUXR |=  (1<<6);    //Timer1 set as 1T mode
	TMOD &= ~(1<<6);    //Timer1 set As Timer
	TMOD &= ~0x30;      //Timer1_16bitAutoReload;
	TH1 = (u8)((65536UL - (FOSC / 4) / BAUD1) / 256);
	TL1 = (u8)((65536UL - (FOSC / 4) / BAUD1) % 256);
	ET1 = 0;    //禁止中断
	TR1  = 1;
	//8位数据, 1位起始位, 1位停止位, 无校验
	SCON = (SCON & 0x3f) | 0x40;    //UART1模式, 0x00: 同步移位输出, 0x40: 8位数据,可变波特率, 0x80: 9位数据,固定波特率, 0xc0: 9位数据,可变波特率
//  PS  = 1;    //高优先级中断
	ES  = 1;    //允许中断
	REN = 1;    //允许接收
	B_TX1_Busy = 0;
	TX1_Cnt = 0;
	RX1_Cnt = 0;
	RS485_RX();
}

//========================================================================
// 函数: void UART1_int (void) interrupt UART1_VECTOR
// 描述: UART1中断函数
// 参数: none
// 返回: none
// 备注: 
//========================================================================
void UART1_int (void) interrupt 4
{
	/*
	在模式0，当串口接收数据第8位结束时,由硬件自动将RI置1,并向主机请求中断，响应中断后,需要主动将RI清零
	在其他模式，串行接收到停止为的中间时刻由硬件自动将RI置1，向CPU发中断申请，响应中断后RI必须由软件清零。
	*/
	//if(RI)
	if((SCON & 1) != 0)
    {
        SCON &= ~1;    //Clear Rx flag
			  //RI = 0;//Clear Rx flag
        RX1_Buffer[RX1_Cnt] = SBUF;
        if(++RX1_Cnt >= 3)  { 
					RX1_Cnt = 0;
					//RI = 0;//Clear Rx flag
					RS485_TX();
				}
    }
		/*
		在模式0，当串口发送数据第8位结束时,由硬件自动将TI置1，并向主机请求中断，响应中断后,需要主动将TI清零
		在其他模式，则在停止位开始发送时由硬件自动将TI置1，向CPU发中断申请，响应中断后TI必须由软件清零。
		*/
    //if(TI)
		if((SCON & 2) != 0)	
    {
			 SCON &= ~2;    //Clear Tx flag
			 //TI = 0;//Clear Rx flag
			 if(++TX1_Cnt >= SendLength)  { 
					B_TX1_Busy = 0;
				 RS485_RX();
				  return;
			 }else{
					SBUF = *(pSendData + TX1_Cnt);
				}
    }
}